==> avenger-0.5.9.tar.gz: avenger-0.5.9/asmtpd/rawnet.h <==
// -*-c++-*-
/* $Id: rawnet.h,v 1.5 2004/12/16 09:59:26 dm Exp $ */

/*
 *
 * Copyright (C) 2004 David Mazieres (dm@uun.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 *
 */

#ifndef _RAWNET_H_
#define _RAWNET_H_ 1

#include "async.h"
#include "qhash.h"
#include "list.h"


inline u_int16_t
cksum (const void *data, size_t len, u_int32_t sum = 0, bool flip = false)
{
  const u_int8_t *dp = static_cast<const u_int8_t *> (data);

  if (reinterpret_cast<u_long> (dp) & 1)
    sum += ntohs (*dp) + cksum (dp + 1, len - 1, 0, true);
  else {
    for (const u_int8_t *ep = dp + (len & ~1); dp < ep;
         dp += sizeof (u_int16_t))
      sum += *reinterpret_cast<const u_int16_t *> (dp);
    if (len & 1)
      sum += ntohs (*dp << 8);
  }

  while (sum > 0xffff)
    sum = (sum & 0xffff) + (sum >> 16);
  return flip ? (sum >> 8) | ((sum & 0xff) << 8) : sum;
}

/* synfp.C */
extern "C" {
#include <pcap.h>
}

struct synfp {
  in_addr addr;
  u_int16_t port;
  pcap_t *pch;
  int hdrlen;
  int fd;

  synfp ();
  ~synfp ();
  bool init (str dev, in_addr addr, int port);
  void close ();
  int getfp (str *fp, sockaddr_in *sinp, sockaddr_in *dsinp = NULL);
  bool sethdrlen (const u_char *ip, const u_char *end);

  static bool pktok (const u_char *ip, const u_char *end);
  static bool ifnames (vec<str> *ifs, in_addr a);
};

template<class T> struct synfp_state {
  struct synfp_entry {
    const sockaddr_in sin;
    T val;
    ihash_entry<synfp_entry> hlink;
    tailq_entry<synfp_entry> tlink;

    synfp_entry (const sockaddr_in &sin, const T &val)
      : sin (sin), val (val) {}
  };

  u_int size;
  ihash<const sockaddr_in, synfp_entry, &synfp_entry::synfp_entry::sin,
	&synfp_entry::synfp_entry::hlink> tab;
  tailq<synfp_entry, &synfp_entry::synfp_entry::tlink> lru;

  void insert (const sockaddr_in &addr, const T &val) {
    synfp_entry *se = tab[addr];
    if (se) {
      lru.remove (se);
      se->val = val;
    }
    else {
      se = New synfp_entry (addr, val);
      tab.insert (se);
    }
    lru.insert_tail (se);
    size++;
  }
  void dealloc (synfp_entry *se) {
    lru.remove (se);
    tab.remove (se);
    delete se;
    size--;
  }
  synfp_state () : size (0) {}
  ~synfp_state () { tab.deleteall (); }
};

class synfp_collect {
  struct tcbs {
    timespec ts;
    cbs cb;
    tcbs (cbs cb) : ts (tsnow), cb (cb) {}
  };

  typedef synfp_state<tcbs>::synfp_entry cbentry_t;
  typedef synfp_state<str>::synfp_entry fpentry_t;

  vec<ref<synfp> > pfv;
  synfp_state<str> fps;
  synfp_state<tcbs> cbs;
  timecb_t *tmo;
  fd_set *fds;

  void input (int i);
  void service ();
  void timeout ();

public:
  timespec delay;
  u_int bufsize;

  synfp_collect (u_int msec, u_int bufsize);
  ~synfp_collect ();
  bool init (const sockaddr_in &sin);
  void lookup (const sockaddr_in &sin, ::cbs cb);
  str lookup (const sockaddr_in &sin);
};

void synfp_test (int argc, char **argv);

/* netpath.C */
struct traceroute;
/* (*cb) (int total_hops, in_addr *addresses, int addresses_size) */
typedef callback<void, int, in_addr *, int>::ref netpathcb_t;
/* hops = 0 => full path
 * hops < 0 => distance from destination
 * hops > 0 => distance from source */
traceroute *netpath (const sockaddr_in *dest, int hops, netpathcb_t);
void netpath_addcb (traceroute *trp, netpathcb_t cb);
void netpath_cancel (traceroute *trp);
void netpath_reset ();
void netpath_test (int argc, char **argv);

/* osguess.C */
const char *synos_guess (str synfp);

#endif /* !_RAWNET_H_ */

==> avenger-0.6.2.tar.gz: avenger-0.6.2/asmtpd/rawnet.h <==
// -*-c++-*-
/* $Id: rawnet.h,v 1.10 2005/05/02 15:53:59 dm Exp $ */

/*
 *
 * Copyright (C) 2004 David Mazieres (dm@uun.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 *
 */

#ifndef _RAWNET_H_
#define _RAWNET_H_ 1

#include "async.h"
#include "qhash.h"
#include "list.h"


inline u_int16_t
cksum (const void *data, size_t len, u_int32_t sum = 0, bool flip = false)
{
  const u_int8_t *dp = static_cast<const u_int8_t *> (data);

  if (reinterpret_cast<u_long> (dp) & 1)
    sum += ntohs (*dp) + cksum (dp + 1, len - 1, 0, true);
  else {
    for (const u_int8_t *ep = dp + (len & ~1); dp < ep;
         dp += sizeof (u_int16_t))
      sum += *reinterpret_cast<const u_int16_t *> (dp);
    if (len & 1)
      sum += ntohs (*dp << 8);
  }

  while (sum > 0xffff)
    sum = (sum & 0xffff) + (sum >> 16);
  return flip ? (sum >> 8) | ((sum & 0xff) << 8) : sum;
}

/* synfp.C */
extern "C" {
#include <pcap.h>
}

struct synfp {
  in_addr addr;
  u_int16_t port;
  pcap_t *pch;
  int hdrlen;
  int fd;

  synfp ();
  ~synfp ();
  bool init (str dev, in_addr addr, int port);
  void close ();
  int getfp (str *fp, sockaddr_in *sinp, sockaddr_in *dsinp = NULL);
  bool sethdrlen (const u_char *ip, const u_char *end);

  static bool pktok (const u_char *ip, const u_char *end);
  static bool ifnames (vec<str> *ifs, in_addr a);
};

template<class T, bool dup> struct synfp_state {
  struct synfp_entry {
    const sockaddr_in sin;
    T val;
    ihash_entry<synfp_entry> hlink;
    tailq_entry<synfp_entry> tlink;

    synfp_entry (const sockaddr_in &sin, const T &val)
      : sin (sin), val (val) {}
  };

  u_int size;
  ihash<const sockaddr_in, synfp_entry, &synfp_entry::synfp_entry::sin,
	&synfp_entry::synfp_entry::hlink> tab;
  tailq<synfp_entry, &synfp_state::synfp_entry::tlink> lru;

  void insert (const sockaddr_in &addr, const T &val) {
    synfp_entry *se = dup ? NULL : tab[addr];
    if (se) {
      lru.remove (se);
      se->val = val;
    }
    else {
      se = New synfp_entry (addr, val);
      tab.insert (se);
      size++;
    }
    lru.insert_tail (se);
  }
  void dealloc (synfp_entry *se) {
    lru.remove (se);
    tab.remove (se);
    delete se;
    size--;
  }
  synfp_state () : size (0) {}
  ~synfp_state () { tab.deleteall (); }
};

class synfp_collect {
  struct tcbs {
    timespec ts;
    cbs cb;
    tcbs (cbs cb) : ts (tsnow), cb (cb) {}
  };

  typedef synfp_state<tcbs, true>::synfp_entry cbentry_t;
  typedef synfp_state<str, false>::synfp_entry fpentry_t;

  vec<ref<synfp> > pfv;
  synfp_state<str, false> fps;
  synfp_state<tcbs, true> cbs;
  timecb_t *tmo;
  fd_set *fds;

  void input (int i);
  void service ();
  void timeout ();

public:
  timespec delay;
  const u_int bufsize;

  synfp_collect (u_int msec, u_int bufsize);
  ~synfp_collect ();
  bool init (const sockaddr_in &sin);
  void lookup (const sockaddr_in &sin, ::cbs cb);
  str lookup (const sockaddr_in &sin);
};

void synfp_test (int argc, char **argv);

/* netpath.C */
struct traceroute;
/* (*cb) (int total_hops, in_addr *addresses, int addresses_size) */
typedef callback<void, int, in_addr *, int>::ref netpathcb_t;
/* hops = 0 => full path
 * hops < 0 => distance from destination
 * hops > 0 => distance from source */
traceroute *netpath (const sockaddr_in *dest, int hops, netpathcb_t);
void netpath_addcb (traceroute *trp, netpathcb_t cb);
void netpath_cancel (traceroute *trp);
void netpath_reset ();
void netpath_test (int argc, char **argv);

/* osguess.C */
const char *synos_guess (str synfp);

#endif /* !_RAWNET_H_ */
